Attribute VB_Name = "Module1"
Option Explicit
'
' Brad Martinez, http://www.mvps.org/ccrp/
'

Declare Function SendMessage Lib "user32" Alias "SendMessageA" _
                            (ByVal hWnd As Long, _
                            ByVal wMsg As Long, _
                            wParam As Any, _
                            lParam As Any) As Long   ' <---

' ===========================================================================
' treeview definitions defined in Commctrl.h

Public Const TV_FIRST = &H1100
Public Const TVM_GETITEM = (TV_FIRST + 12)
Public Const TVM_SETITEM = (TV_FIRST + 13)

Public Type TVITEM   ' was TV_ITEM
  mask As Long
  hItem As Long
  state As TVItemStates
  stateMask As TVItemStates
  pszText As String
  cchTextMax As Long
  iImage As Long
  iSelectedImage As Long
  cChildren As Long
  lParam As Long
End Type

' TVITEM mask value
Public Const TVIF_STATE = &H8

Public Enum TVItemStates
  TVIS_FOCUSED = &H1   ' (focus rect) read-only, no more than one item has this
  TVIS_SELECTED = &H2   ' (highlighted) read-write, more than one item can have this
  TVIS_CUT = &H4
  TVIS_DROPHILITED = &H8
  TVIS_BOLD = &H10
  TVIS_EXPANDED = &H20
  TVIS_EXPANDEDONCE = &H40
  
#If (WIN32_IE >= &H300) Then
  TVIS_EXPANDPARTIAL = &H80
#End If
  
  TVIS_OVERLAYMASK = &HF00
  TVIS_STATEIMAGEMASK = &HF000
  TVIS_USERMASK = &HF000
End Enum
'

' Returns the specified item state of the specified treeview item.

'   hwndTV          - treeview's window handle
'   hItem               - handle of the item

Public Static Function GetTVItemState(hwndTV As Long, hItem As Long) As TVItemStates
  Dim tvi As TVITEM
  
  tvi.hItem = hItem
  tvi.mask = TVIF_STATE
  
  If TreeView_GetItem(hwndTV, tvi) Then
    GetTVItemState = tvi.state
  End If

End Function

' Sets the specified item state of the specified item.
' Note that *no* corresponding TVN_ notification is generated
' (TVIF_STATE acts only on the item's "state", not the item itself).

'   hwndTV          - treeview's window handle
'   hItem               - handle of the item
'   dwNewState   - state bits to set
'   fAdd                - flag indicating whether the state bits are added or removed,

' Returns True if the new state was successfully set, False otherwise.
' (some of the time...??)

Public Function SetTVItemState(hwndTV As Long, _
                                                   hItem As Long, _
                                                   dwNewState As TVItemStates, _
                                                   fAdd As Boolean) As Boolean
  Dim tvi As TVITEM
  
  tvi.hItem = hItem
  tvi.mask = TVIF_STATE
  tvi.state = (fAdd And dwNewState)
  tvi.stateMask = dwNewState
  
  ' Old docs say returns 0 on success, -1 on failure. New docs say
  ' returns TRUE if successful, or FALSE otherwise. We'll go new...
  SetTVItemState = TreeView_SetItem(hwndTV, tvi)

End Function

' treeview macros defined in Commctrl.h

' Retrieves some or all of a tree-view item's attributes.
' Returns TRUE if successful or FALSE otherwise.

Public Function TreeView_GetItem(hWnd As Long, pitem As TVITEM) As Boolean
  TreeView_GetItem = SendMessage(hWnd, TVM_GETITEM, 0, pitem)
End Function

' Sets some or all of a tree-view item's attributes.
' Returns TRUE if successful or FALSE otherwise.

Public Function TreeView_SetItem(hWnd As Long, pitem As TVITEM) As Boolean
  TreeView_SetItem = SendMessage(hWnd, TVM_SETITEM, 0, pitem)
End Function
